/* This file left intentionally (almost) blank

   Landslide always adds a print.css and a screen.css, but they
   are not needed in impress.js, so this theme leaves them blank,
   except for hiding some things you want to hide.

   You can modify these files in your own fork, or you can add
   css-files in the landslide configuration file.
   See https://github.com/adamzap/landslide#presentation-configuration

*/

.impress-supported .fallback-message,
.step .notes {
    display: none;
}

.step {
    width: 800px;
}


/* Help popup */

#hovercraft-help {
    background: none repeat scroll 0 0 rgba(0, 0, 0, 0.7);
    color: #EEEEEE;
    font-size: 70%;
    left: 2em;
    bottom: 2em;
    width: 28em;
    border-radius: 1em;
    padding: 1em;
    position: fixed;
    right: 0;
    text-align: center;
    z-index: 100;
    display: block;
    font-family: Verdana, Arial, Sans;
}

.impress-enabled #hovercraft-help.hide {
    display: none;
}

#hovercraft-help.disabled {
    display: none;
}

/* Slide numbers */

div#slide-number
{
    bottom: 1em;
    font-size: 5vh;
    position: absolute;
    right: 2.5em;
    text-align: right;
}
