@import url(http://fonts.googleapis.com/css?family=Libre+Baskerville|Racing+Sans+One|Satisfy);

body {
    font-size: 200%;
    text-align: center;
    font-family: Libre Baskerville;
}

p {
    font-size: 125%;
    font-weight: normal;
    margin-bottom: 0.3em;
    margin-top: 0.3em;
}

em {
    font-family: Satisfy;
    font-size: 142%;
}

strong {
    font-family: Racing Sans One;
    font-size: 127%;
    letter-spacing: -2px;
}

h1, h2, h3, h4 {
    font-weight: normal;
    margin-bottom: 0.3em;
    margin-top: 0.3em;
}

ul {
    list-style-position: inside;
}

li {
    font-size: 125%;
    font-family: serif;
    font-weight: normal;
    margin-bottom: 0.3em;
    margin-top: 0.3em;
    padding-left: 0.3em;
    line-height: 1.8em;
    list-style-type: circle;
}

a {
    text-decoration: none;
    color: black;
}

#ThreeD H1 {
    font-family: Racing Sans One;
    font-size: 300%;
}

#thequestion p {
    font-family: Racing Sans One;
}
